@@ -5,39 +5,34 @@
 */
 
 #include "fuzzers.h"
+
 #include <botan/asn1_print.h>
 #include <fstream>
 
-class ASN1_Parser final : public Botan::ASN1_Formatter
-   {
+class ASN1_Parser final : public Botan::ASN1_Formatter {
    public:
       ASN1_Parser() : Botan::ASN1_Formatter(true, 64) {}
 
    protected:
-      std::string format(Botan::ASN1_Tag, Botan::ASN1_Tag, size_t, size_t,
-                         const std::string&) const override
-         {
+      std::string format(Botan::ASN1_Type, Botan::ASN1_Class, size_t, size_t, std::string_view) const override {
          return "";
-         }
+      }
 
-      std::string format_bin(Botan::ASN1_Tag, Botan::ASN1_Tag,
-                             const std::vector<uint8_t>&) const override
-         {
+      std::string format_bin(Botan::ASN1_Type, Botan::ASN1_Class, const std::vector<uint8_t>&) const override {
          return "";
-         }
-   };
+      }
 
-void fuzz(const uint8_t in[], size_t len)
-   {
-   try
-      {
+      std::string format_bn(const Botan::BigInt&) const override { return ""; }
+};
+
+void fuzz(std::span<const uint8_t> in) {
+   try {
       /*
       * Here we use an uninitialized ofstream so the fuzzer doesn't spend time
       * on actual output formatting, no memory is allocated, etc.
       */
       std::ofstream out;
       ASN1_Parser printer;
-      printer.print_to_stream(out, in, len);
-      }
-   catch(Botan::Exception& e) { }
-   }
+      printer.print_to_stream(out, in.data(), in.size());
+   } catch(Botan::Exception& e) {}
+}
