@@ -5,18 +5,17 @@
 */
 
 #include "fuzzers.h"
-#include <botan/x509cert.h>
+
 #include <botan/data_src.h>
+#include <botan/x509cert.h>
 
-void fuzz(const uint8_t in[], size_t len)
-   {
-   if(len > max_fuzzer_input_size)
+void fuzz(std::span<const uint8_t> in) {
+   if(in.size() > max_fuzzer_input_size) {
       return;
-
-   try
-      {
-      Botan::DataSource_Memory input(in, len);
-      Botan::X509_Certificate cert(input);
-      }
-   catch(Botan::Exception& e) { }
    }
+
+   try {
+      Botan::DataSource_Memory input(in);
+      Botan::X509_Certificate cert(input);
+   } catch(Botan::Exception& e) {}
+}
