@@ -6,6 +6,7 @@
 */
 
 #include <botan/pipe.h>
+
 #include <botan/exceptn.h>
 #include <unistd.h>
 
@@ -14,42 +15,39 @@ namespace Botan {
 /*
 * Write data from a pipe into a Unix fd
 */
-int operator<<(int fd, Pipe& pipe)
-   {
+int operator<<(int fd, Pipe& pipe) {
    secure_vector<uint8_t> buffer(BOTAN_DEFAULT_BUFFER_SIZE);
-   while(pipe.remaining())
-      {
+   while(pipe.remaining()) {
       size_t got = pipe.read(buffer.data(), buffer.size());
       size_t position = 0;
-      while(got)
-         {
+      while(got) {
          ssize_t ret = ::write(fd, &buffer[position], got);
-         if(ret < 0)
+         if(ret < 0) {
             throw Stream_IO_Error("Pipe output operator (unixfd) has failed");
+         }
 
          position += static_cast<size_t>(ret);
          got -= static_cast<size_t>(ret);
-         }
       }
-   return fd;
    }
+   return fd;
+}
 
 /*
 * Read data from a Unix fd into a pipe
 */
-int operator>>(int fd, Pipe& pipe)
-   {
+int operator>>(int fd, Pipe& pipe) {
    secure_vector<uint8_t> buffer(BOTAN_DEFAULT_BUFFER_SIZE);
-   while(true)
-      {
+   while(true) {
       ssize_t ret = ::read(fd, buffer.data(), buffer.size());
-      if(ret < 0)
+      if(ret < 0) {
          throw Stream_IO_Error("Pipe input operator (unixfd) has failed");
-      else if(ret == 0)
+      } else if(ret == 0) {
          break;
-      pipe.write(buffer.data(), static_cast<size_t>(ret));
       }
-   return fd;
+      pipe.write(buffer.data(), static_cast<size_t>(ret));
    }
-
+   return fd;
 }
+
+}  // namespace Botan
