@@ -10,6 +10,13 @@
 
 #include <botan/symkey.h>
 #include <chrono>
+#include <string>
+#include <string_view>
+
+/*
+* This entire interface is deprecated. Use the interface in pwdhash.h
+*/
+BOTAN_DEPRECATED_HEADER("pbkdf.h")
 
 namespace Botan {
 
@@ -21,8 +28,7 @@ namespace Botan {
 * Starting in 2.8 this functionality is also offered by PasswordHash.
 * The PBKDF interface may be removed in a future release.
 */
-class BOTAN_PUBLIC_API(2,0) PBKDF
-   {
+class BOTAN_PUBLIC_API(2, 0) PBKDF {
    public:
       /**
       * Create an instance based on a name
@@ -31,34 +37,38 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @param provider provider implementation to choose
       * @return a null pointer if the algo/provider combination cannot be found
       */
-      static std::unique_ptr<PBKDF> create(const std::string& algo_spec,
-                                           const std::string& provider = "");
+      BOTAN_DEPRECATED("Use PasswordHashFamily + PasswordHash")
+      static std::unique_ptr<PBKDF> create(std::string_view algo_spec, std::string_view provider = "");
 
       /**
       * Create an instance based on a name, or throw if the
       * algo/provider combination cannot be found. If provider is
       * empty then best available is chosen.
       */
-      static std::unique_ptr<PBKDF>
-         create_or_throw(const std::string& algo_spec,
-                         const std::string& provider = "");
+      BOTAN_DEPRECATED("Use PasswordHashFamily + PasswordHash")
+      static std::unique_ptr<PBKDF> create_or_throw(std::string_view algo_spec, std::string_view provider = "");
 
       /**
       * @return list of available providers for this algorithm, empty if not available
       */
-      static std::vector<std::string> providers(const std::string& algo_spec);
+      static std::vector<std::string> providers(std::string_view algo_spec);
 
       /**
       * @return new instance of this same algorithm
       */
-      virtual PBKDF* clone() const = 0;
+      virtual std::unique_ptr<PBKDF> new_object() const = 0;
+
+      /**
+      * @return new instance of this same algorithm
+      */
+      PBKDF* clone() const { return this->new_object().release(); }
 
       /**
       * @return name of this PBKDF
       */
       virtual std::string name() const = 0;
 
-      virtual ~PBKDF() {}
+      virtual ~PBKDF() = default;
 
       /**
       * Derive a key from a passphrase for a number of iterations
@@ -75,9 +85,11 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       *        run until msec milliseconds has passed.
       * @return the number of iterations performed
       */
-      virtual size_t pbkdf(uint8_t out[], size_t out_len,
-                           const std::string& passphrase,
-                           const uint8_t salt[], size_t salt_len,
+      virtual size_t pbkdf(uint8_t out[],
+                           size_t out_len,
+                           std::string_view passphrase,
+                           const uint8_t salt[],
+                           size_t salt_len,
                            size_t iterations,
                            std::chrono::milliseconds msec) const = 0;
 
@@ -91,9 +103,11 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @param salt_len length of salt in bytes
       * @param iterations the number of iterations to use (use 10K or more)
       */
-      void pbkdf_iterations(uint8_t out[], size_t out_len,
-                            const std::string& passphrase,
-                            const uint8_t salt[], size_t salt_len,
+      void pbkdf_iterations(uint8_t out[],
+                            size_t out_len,
+                            std::string_view passphrase,
+                            const uint8_t salt[],
+                            size_t salt_len,
                             size_t iterations) const;
 
       /**
@@ -108,11 +122,13 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       *        run until msec milliseconds has passed.
       * @param iterations set to the number iterations executed
       */
-      void pbkdf_timed(uint8_t out[], size_t out_len,
-                         const std::string& passphrase,
-                         const uint8_t salt[], size_t salt_len,
-                         std::chrono::milliseconds msec,
-                         size_t& iterations) const;
+      void pbkdf_timed(uint8_t out[],
+                       size_t out_len,
+                       std::string_view passphrase,
+                       const uint8_t salt[],
+                       size_t salt_len,
+                       std::chrono::milliseconds msec,
+                       size_t& iterations) const;
 
       /**
       * Derive a key from a passphrase for a number of iterations.
@@ -124,10 +140,8 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @param iterations the number of iterations to use (use 10K or more)
       * @return the derived key
       */
-      secure_vector<uint8_t> pbkdf_iterations(size_t out_len,
-                                           const std::string& passphrase,
-                                           const uint8_t salt[], size_t salt_len,
-                                           size_t iterations) const;
+      secure_vector<uint8_t> pbkdf_iterations(
+         size_t out_len, std::string_view passphrase, const uint8_t salt[], size_t salt_len, size_t iterations) const;
 
       /**
       * Derive a key from a passphrase, running until msec time has elapsed.
@@ -142,10 +156,11 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @return the derived key
       */
       secure_vector<uint8_t> pbkdf_timed(size_t out_len,
-                                      const std::string& passphrase,
-                                      const uint8_t salt[], size_t salt_len,
-                                      std::chrono::milliseconds msec,
-                                      size_t& iterations) const;
+                                         std::string_view passphrase,
+                                         const uint8_t salt[],
+                                         size_t salt_len,
+                                         std::chrono::milliseconds msec,
+                                         size_t& iterations) const;
 
       // Following kept for compat with 1.10:
 
@@ -157,13 +172,10 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @param salt_len length of salt in bytes
       * @param iterations the number of iterations to use (use 10K or more)
       */
-      OctetString derive_key(size_t out_len,
-                             const std::string& passphrase,
-                             const uint8_t salt[], size_t salt_len,
-                             size_t iterations) const
-         {
-         return pbkdf_iterations(out_len, passphrase, salt, salt_len, iterations);
-         }
+      OctetString derive_key(
+         size_t out_len, std::string_view passphrase, const uint8_t salt[], size_t salt_len, size_t iterations) const {
+         return OctetString(pbkdf_iterations(out_len, passphrase, salt, salt_len, iterations));
+      }
 
       /**
       * Derive a key from a passphrase
@@ -172,14 +184,13 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @param salt a randomly chosen salt
       * @param iterations the number of iterations to use (use 10K or more)
       */
-      template<typename Alloc>
+      template <typename Alloc>
       OctetString derive_key(size_t out_len,
-                             const std::string& passphrase,
+                             std::string_view passphrase,
                              const std::vector<uint8_t, Alloc>& salt,
-                             size_t iterations) const
-         {
-         return pbkdf_iterations(out_len, passphrase, salt.data(), salt.size(), iterations);
-         }
+                             size_t iterations) const {
+         return OctetString(pbkdf_iterations(out_len, passphrase, salt.data(), salt.size(), iterations));
+      }
 
       /**
       * Derive a key from a passphrase
@@ -191,13 +202,13 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @param iterations is set to the number of iterations used
       */
       OctetString derive_key(size_t out_len,
-                             const std::string& passphrase,
-                             const uint8_t salt[], size_t salt_len,
+                             std::string_view passphrase,
+                             const uint8_t salt[],
+                             size_t salt_len,
                              std::chrono::milliseconds msec,
-                             size_t& iterations) const
-         {
-         return pbkdf_timed(out_len, passphrase, salt, salt_len, msec, iterations);
-         }
+                             size_t& iterations) const {
+         return OctetString(pbkdf_timed(out_len, passphrase, salt, salt_len, msec, iterations));
+      }
 
       /**
       * Derive a key from a passphrase using a certain amount of time
@@ -207,16 +218,15 @@ class BOTAN_PUBLIC_API(2,0) PBKDF
       * @param msec is how long to run the PBKDF
       * @param iterations is set to the number of iterations used
       */
-      template<typename Alloc>
+      template <typename Alloc>
       OctetString derive_key(size_t out_len,
-                             const std::string& passphrase,
+                             std::string_view passphrase,
                              const std::vector<uint8_t, Alloc>& salt,
                              std::chrono::milliseconds msec,
-                             size_t& iterations) const
-         {
-         return pbkdf_timed(out_len, passphrase, salt.data(), salt.size(), msec, iterations);
-         }
-   };
+                             size_t& iterations) const {
+         return OctetString(pbkdf_timed(out_len, passphrase, salt.data(), salt.size(), msec, iterations));
+      }
+};
 
 /*
 * Compatibility typedef
@@ -229,18 +239,15 @@ typedef PBKDF S2K;
 * @param provider the provider to use
 * @return pointer to newly allocated object of that type
 */
-inline PBKDF* get_pbkdf(const std::string& algo_spec,
-                        const std::string& provider = "")
-   {
+BOTAN_DEPRECATED("Use PasswordHashFamily + PasswordHash")
+inline PBKDF* get_pbkdf(std::string_view algo_spec, std::string_view provider = "") {
    return PBKDF::create_or_throw(algo_spec, provider).release();
-   }
-
-inline PBKDF* get_s2k(const std::string& algo_spec)
-   {
-   return get_pbkdf(algo_spec);
-   }
-
-
 }
 
+BOTAN_DEPRECATED("Use PasswordHashFamily + PasswordHash") inline PBKDF* get_s2k(std::string_view algo_spec) {
+   return PBKDF::create_or_throw(algo_spec).release();
+}
+
+}  // namespace Botan
+
 #endif
