@@ -16,8 +16,7 @@ namespace Botan {
 * On x86, the RDRAND instruction is used.
 * on POWER, the DARN instruction is used.
 */
-class BOTAN_PUBLIC_API(2,15) Processor_RNG final : public Hardware_RNG
-   {
+class BOTAN_PUBLIC_API(2, 15) Processor_RNG final : public Hardware_RNG {
    public:
       /**
       * Constructor will throw if CPU does not have RDRAND bit set
@@ -30,23 +29,20 @@ class BOTAN_PUBLIC_API(2,15) Processor_RNG final : public Hardware_RNG
       static bool available();
 
       bool accepts_input() const override { return false; }
+
       bool is_seeded() const override { return true; }
 
-      void randomize(uint8_t out[], size_t out_len) override;
-
-      /*
-      * No way to provide entropy to RDRAND generator, so add_entropy is ignored
-      */
-      void add_entropy(const uint8_t[], size_t) override;
-
       /*
       * No way to reseed processor provided generator, so reseed is ignored
       */
       size_t reseed(Entropy_Sources&, size_t, std::chrono::milliseconds) override;
 
       std::string name() const override;
-   };
 
-}
+   private:
+      void fill_bytes_with_input(std::span<uint8_t> out, std::span<const uint8_t> in) override;
+};
+
+}  // namespace Botan
 
 #endif
