@@ -11,28 +11,23 @@
 
 namespace Botan {
 
-void Semaphore::release(size_t n)
-   {
-   for(size_t i = 0; i != n; ++i)
-      {
+void Semaphore::release(size_t n) {
+   for(size_t i = 0; i != n; ++i) {
       std::lock_guard<std::mutex> lock(m_mutex);
 
-      if(m_value++ < 0)
-         {
+      if(m_value++ < 0) {
          ++m_wakeups;
          m_cond.notify_one();
-         }
       }
    }
-
-void Semaphore::acquire()
-   {
-   std::unique_lock<std::mutex> lock(m_mutex);
-   if(m_value-- <= 0)
-       {
-       m_cond.wait(lock, [this] { return m_wakeups > 0; });
-       --m_wakeups;
-       }
-   }
-
 }
+
+void Semaphore::acquire() {
+   std::unique_lock<std::mutex> lock(m_mutex);
+   if(m_value-- <= 0) {
+      m_cond.wait(lock, [this] { return m_wakeups > 0; });
+      --m_wakeups;
+   }
+}
+
+}  // namespace Botan
